
# q-expansion of newform 360.2.b.d, downloaded from the LMFDB on 13 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 360
weight = 2
poly_data = [8, -8, 2, 0, 1, -2, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0], 1], [[-8, 6, 2, 1, 0, 1], 4], [[-4, 2, -2, 1, 0, 1], 4], [[4, 2, -2, -1, 0, -1], 4], [[-8, 2, 0, 1, -2, 1], 4], [[16, -6, 0, 1, 2, -3], 4]]

hecke_ring_character_values = [[271, [-1, 0, 0, 0, 0, 0]], [181, [-1, 0, 0, 0, 0, 0]], [281, [-1, 0, 0, 0, 0, 0]], [217, [1, 0, 0, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0, -1, 0], [0, 0, 0, 0, 0, 0], [1, 0, 0, 0, 0, 0], [0, 0, 1, 0, 0, 0], [-1, -1, 1, -1, -2, 0], [1, 1, 1, 1, 2, 0], [-1, -1, 2, 1, 0, 2], [3, -1, 0, -1, 2, 0], [1, -1, 0, 1, 0, -2], [-2, 0, 0, -2, 2, 2], [1, 1, -2, -1, 0, -2], [1, 1, -1, -1, 0, -2], [0, 0, -1, 2, 2, 2], [-2, -2, 0, 0, 2, -2], [1, 1, 0, 3, -4, -2], [2, -2, 0, 0, 2, -2], [1, 1, 1, -1, 0, -2], [2, 2, -2, 2, 4, 0], [0, 0, 0, -2, 2, 2], [-8, 0, 0, 2, -2, -2], [-2, 0, 0, 4, -4, -4], [-3, -3, -2, -1, -4, 2], [2, 2, -2, -2, 0, -4], [0, 0, 3, 2, 2, 2], [0, 2, 0, 4, -6, -2], [-6, 4, 0, 0, -4, 4], [0, 0, 7, 2, 2, 2], [0, 0, 2, 0, 0, 0], [-2, -2, -2, -2, -4, 0], [3, 3, -4, 1, 4, -2], [0, 0, -9, 2, 2, 2], [-3, -3, -7, -3, -6, 0], [-1, -1, -8, -3, -4, -2], [-3, 3, 0, -3, 0, 6], [6, 0, 0, -2, 2, 2], [-1, -1, 0, -7, -8, -6], [-1, -1, 1, -3, -4, -2], [-10, 2, 0, 2, -4, 0], [-7, 3, 0, -3, 0, 6], [14, -2, 0, 0, 2, -2], [-1, -1, 3, 3, 2, 4], [0, 0, -6, 4, 4, 4], [4, 0, 0, -8, 8, 8], [-8, -2, 0, -4, 6, 2], [-8, -2, 0, -2, 4, 0], [1, 1, 8, 3, 4, 2], [5, 3, 0, 5, -8, -2], [-2, -2, -3, 2, 0, 4], [2, 2, -8, 2, 4, 0], [0, 0, 4, -4, -4, -4], [-1, -1, 6, -3, -4, -2], [10, 2, 0, 2, -4, 0], [6, 2, 0, -2, 0, 4], [-5, -5, 9, 1, -4, 6], [-3, -3, 4, -5, -8, -2], [-3, -1, 0, -3, 4, 2], [-10, 0, 0, 6, -6, -6], [3, 3, -12, -3, 0, -6], [1, 1, -7, -5, -4, -6], [6, 6, -1, 2, 8, -4], [20, 0, 0, 2, -2, -2], [-6, 0, 0, 0, 0, 0], [10, 2, 0, -2, 0, 4], [-22, -2, 0, -2, 4, 0], [12, -2, 0, 4, -2, -6], [-4, -2, 0, 2, 0, -4], [-5, -1, 0, 3, -2, -4], [-2, -4, 0, 4, 0, -8], [4, 4, 4, 8, 12, 4], [6, 6, -2, 2, 8, -4], [3, 3, 2, 1, 4, -2], [-12, -4, 0, -2, 6, -2], [-2, -2, -9, 0, -2, 2], [-3, -3, -13, -5, -8, -2], [1, -1, 0, 5, -4, -6], [9, 1, 0, -5, 4, 6], [-2, -4, 0, 4, 0, -8], [1, 1, -5, -7, -6, -8], [2, 2, -3, 0, 2, -2], [2, -4, 0, 0, 4, -4], [1, 1, -5, -1, 0, -2], [0, 0, 8, 4, 4, 4], [8, -4, 0, -2, 6, -2], [6, 0, 0, -4, 4, 4], [1, 1, 16, 3, 4, 2], [-2, -2, -16, -2, -4, 0], [0, 0, 3, 0, 0, 0], [0, 6, 0, 4, -10, 2], [10, 4, 0, -6, 2, 10], [2, 2, -19, 0, 2, -2], [0, 0, 10, 4, 4, 4], [8, 0, 0, 6, -6, -6], [4, 4, 3, 2, 6, -2], [1, 1, 1, -5, -4, -6], [-7, 5, 0, -3, -2, 8], [-19, 5, 0, -1, -4, 6], [-6, -4, 0, -8, 12, 4], [0, 0, 5, 4, 4, 4], [8, 4, 0, -4, 0, 8], [-4, -4, 6, 4, 0, 8], [-6, -2, 0, -8, 10, 6], [10, -2, 0, -4, 6, 2], [-4, -4, 12, -4, -8, 0], [-2, -2, -15, -4, -6, -2], [-9, 3, 0, 7, -10, -4], [4, 2, 0, 0, -2, 2], [-4, -4, 2, -8, -12, -4], [-7, -7, 0, -1, -8, 6], [-16, 4, 0, 2, -6, 2], [-8, 8, 0, 4, -12, 4], [-4, -4, 1, 4, 0, 8], [3, 3, 5, -1, 2, -4], [7, 7, -6, 5, 12, -2], [-9, -7, 0, -9, 16, 2], [-1, -1, 18, 5, 4, 6], [6, 6, 11, 6, 12, 0], [-22, 2, 0, 2, -4, 0], [-15, 1, 0, 7, -8, -6], [4, 6, 0, -6, 0, 12], [7, 7, -9, -3, 4, -10], [0, 0, -14, 0, 0, 0], [-2, 8, 0, 8, -16, 0], [-8, -2, 0, -2, 4, 0], [2, 2, -22, 2, 4, 0], [-1, -5, 0, -1, 6, -4], [-2, -4, 0, 0, 4, -4], [-4, -4, 20, 4, 0, 8], [14, -2, 0, -8, 10, 6], [-8, -8, 9, -6, -14, 2], [1, 1, -11, -5, -4, -6], [-1, 3, 0, 3, -6, 0], [-23, 1, 0, 7, -8, -6], [-3, -3, -14, -1, -4, 2], [1, 1, -23, 3, 4, 2], [0, 0, -13, -4, -4, -4], [0, 8, 0, -4, -4, 12], [-24, -2, 0, -6, 8, 4], [-4, -4, 0, 12, -8, -16], [-2, -2, 0, -4, 6, 2], [10, 10, 5, 6, 16, -4], [-23, -1, 0, -3, 4, 2], [26, 0, 0, -6, 6, 6], [10, 10, -7, 2, 12, -8], [10, 10, -4, -2, 8, -12], [-4, -4, 26, 4, 0, 8], [12, 4, 0, 10, -14, -6], [3, 3, -15, 3, 6, 0], [-3, -3, 14, 3, 0, 6], [25, -1, 0, 5, -4, -6], [3, 9, 0, 11, -20, -2], [-3, -3, 3, 7, 4, 10], [4, 4, -15, -2, 2, -6], [-14, -2, 0, 2, 0, -4], [-15, -5, 0, -3, 8, -2], [8, -4, 0, 10, -6, -14], [-30, 2, 0, 2, -4, 0], [-7, -7, 6, -5, -12, 2], [-4, -4, 19, 0, -4, 4], [10, 4, 0, -4, 0, 8], [18, -8, 0, 4, 4, -12], [-2, -2, -24, -6, -8, -4], [-3, -3, -8, 3, 0, 6], [-10, -10, 19, -2, -12, 8], [1, 1, 23, 7, 8, 6], [-1, -1, 18, -3, -4, -2], [3, -5, 0, 1, 4, -6], [-3, -3, 8, 3, 0, 6], [13, 13, 3, 5, 18, -8]]
